local shader = Shader.new("blur/vShader", "blur/fShader", 0, {
	{name="vMatrix",type=Shader.CMATRIX,sys=Shader.SYS_WVP,vertex=true},
	{name="fColor",type=Shader.CFLOAT4,sys=Shader.SYS_COLOR,vertex=false},
	{name="fTexture",type=Shader.CTEXTURE,vertex=false},
	{name="fResolution",type=Shader.CFLOAT2,vertex=false},
	{name="fTexelSize",type=Shader.CFLOAT2,vertex=false},
	{name="fDir",type=Shader.CFLOAT2,vertex=false},
},
{
	{name="vVertex",type=Shader.DFLOAT,mult=3,slot=0,offset=0},
	{name="vColor",type=Shader.DUBYTE,mult=4,slot=1,offset=0},
	{name="vTexCoord",type=Shader.DFLOAT,mult=2,slot=2,offset=0},
})

assert(shader:isValid(), "Invalid shader.")

GShape = Core.class(Path2D)

function GShape:init(name, ...)
	self.name = name
	self.needToUpdate = true
	self[name](self, ...)
	
	self.blurName = blurName
	self.blurShader = shader
	self:initShader()
	self:updateBlur()
end
--
function GShape:initShader()
	local texw, texh = self:getSize()
	shader:setConstant("fResolution",Shader.CFLOAT2,1,{texw, texh})
	shader:setConstant("fDir",Shader.CFLOAT2,1,{0,1})
	
	self.blurBG = RenderTarget.new(texw,texh,false)
	self.blurIMG = Bitmap.new(self.blurBG)
	self.blurIMG:setShader(self.blurShader)
	
	self.RT = RenderTarget.new(texw-2,texh-4,true)
	
	if (TEMP_FIX) then 
		self.blured = Bitmap.new(self.RT)
		self.blured:setShader(self.blurShader)
		self:addChild(self.blured)
	else
		self:setTexture(self.RT)
		self:setShader(shader)
	end
end
--
function GShape:renderBG()
	local x,y = self:getPosition()
	local offsetX, offsetY = 0, 0
	
	-- circle is centerd, so we need to offset drawing position by its radius
	if (self.name == "circle") then 
		offsetX = self.r
		offsetY = self.r
	end
	
	local par = self:getParent()
	if (not par) then return end
	local ind = par:getChildIndex(self)
	local l = par:getNumChildren()
	print("Hide from " .. ind .. " to "..l)
	for i = ind, l do 
		local c = par:getChildAt(i)
		c:setVisible(false)
	end
	self.blurBG:draw(stage, -x+offsetX, -y+offsetY)
	for i = ind, l do 
		local c = par:getChildAt(i)
		c:setVisible(true)
	end
end
--
function GShape:updateBlur()
	if (self.needToUpdate) then 
		self:forceUpdate()
	end
end
--
function GShape:forceUpdate()
	self:renderBG()	
	-- vertical blur
	shader:setConstant("fDir",Shader.CFLOAT2,1,{0,1})
	self.RT:draw(self.blurIMG)
	-- horizontal blur
	shader:setConstant("fDir",Shader.CFLOAT2,1,{1,0})
end
--
function GShape:circle(r)
	self.r = r
	local ms="MAAZ"
	local mp={-r,0, r,r,0,0,0,r,0, r,r,0,0,0,-r,0}
	self:setPath(ms,mp)
	
	self:setFillColor(0xffffff, 1)
	self:setLineColor(0, 1)
	self:setLineThickness(2, 1)
end
--
function GShape:rect(w, h)
	local ms="MHVHVZ"
	local mp={0,0, w, h, 0, 0}
	self:setPath(ms,mp)
	
	self:setFillColor(0xffffff, 1)
	self:setLineColor(0, 1)
	self:setLineThickness(4, 1)
end
--
function GShape:rrect(w,h,r)
	local ms="MALALALAZ"
	local mp = {0,r, r,r,0,0,1,r,0, w-r,0, r,r,0,0,1,w,r, w,h-r, r,r,0,0,1,w-r,h, r,h, r,r,0,0,1,0,h-r}
	self:setPath(ms,mp)
	
	self:setFillColor(0xffffff, 1)
	self:setLineColor(0, 1)
	self:setLineThickness(4, 1)
end
--
function GShape:setPosition(x, y)
	Path2D.setPosition(self, x, y)
	self.needToUpdate = true
end